# -*- coding: utf-8 -*-
"""
Views for newsletter subscription
"""

try:
    from django.urls import reverse_lazy
except ImportError:
    from django.core.urlresolvers import reverse_lazy
from django.views.generic.edit import FormView
from django.views.generic.base import TemplateView
from django.http import JsonResponse

from newsletter_subscriptions.forms import SubscriptionForm
from newsletter_subscriptions.helpers import helper_finder, SubscriptionFormHelper


# Ajax form based on the django docs example.
# https://docs.djangoproject.com/en/dev/topics/class-based-views/generic-editing/#ajax-example
# https://docs.djangoproject.com/en/dev/topics/class-based-views/mixins/#more-than-just-html


class SubscribeFormView(FormView):
    template_name = 'newsletter_subscriptions/form.html'
    form_class = SubscriptionForm
    form_helper_class = SubscriptionFormHelper
    subscribe_object = None
    success_url = reverse_lazy('newsletter-subscribe-sent')

    def get_form_helper_class(self):
        return helper_finder(self.form_helper_class)

    def get_form_helper(self):
        helper_class = self.get_form_helper_class()
        kwargs = self.get_form_helper_kwargs()
        return helper_class(**kwargs)

    def get_form_helper_kwargs(self):
        return {}

    def get_context_data(self, **kw):
        context = super(SubscribeFormView, self).get_context_data(**kw)
        context.update({
            'form_helper': self.get_form_helper(),
        })
        return context

    def render_to_json_response(self, context, status=200):
        return JsonResponse(context, status=status)

    def form_valid(self, form):
        subscribe_object = form.save()
        if self.request.is_ajax():
            return self.render_to_json_response({
                'pk': subscribe_object.pk,
            })
        return super(SubscribeFormView, self).form_valid(form)

    def form_invalid(self, form):
        if self.request.is_ajax():
            return self.render_to_json_response(form.errors, status=400)
        return super(SubscribeFormView, self).form_invalid(form)


class SubscribeConfirmView(TemplateView):
    template_name = 'newsletter_subscriptions/form_sent.html'
